from .registro0 import CNAB400Registro0
from .registro1 import CNAB400Registro1
from .registro2 import CNAB400Registro2
from .registro9 import CNAB400Registro9

__all__ = [
    'CNAB400Registro0',
    'CNAB400Registro1',
    'CNAB400Registro2',
    'CNAB400Registro9',
]
